#!/usr/bin/env python3

"""
Tool for generating systems using Savitri software.
Copyright 2022 Protolab LTDA, Brazil
"""

__author__ = 'João Gabriel Santos'
__copyright__ = 'Copyright 2022 Protolab LTDA'
__version__ = '0.0.6'



from dataclasses import dataclass, field
from savitrigen.guideline import (
    check_module_naming,
    check_field_naming
)

@dataclass
class Module(object):
    fields:dict
    _fields:dict = field(init=False, repr=False)

    unicon:str = ''
    route:bool = False
    report:bool = False
    presets:list = None

    """Those below are unused"""
    documentation:str = 'undocumented'
    translation:dict = None

    @property
    def fields(self) -> dict:
        ms = dict()

        for k, v in self._fields.items():
            ms[k] = {
                f_k: f_v
                for f_k, f_v in v.items()
                if f_v is not None
            }

        return ms

    @fields.setter
    def fields(self, value:dict) -> None:
        self._fields = value

@dataclass
class Field(object):
    label:str
    type:str

    description:str = None

    required:bool = False
    readonly:bool = False
    values:list = None


@dataclass
class BackendConfig(object):
    entities:dict
    _entities:dict = field(init=False, repr=False)

    plugins:list[str] = None

    @property
    def entities(self) -> dict:
        return self._entities

    @entities.setter
    def entities(self, value:dict) -> None:
        ms = dict()

        for k, v in value.items():
            ms[k] = module = Module(**v)
            check_module_naming(k)

            for f_k, f_v in module.fields.items():
                module.fields[f_k] = Field(**f_v).__dict__
                check_field_naming(f_k)

        self._entities = ms



from .rules import (
    MODULE_NAMING,
    FIELD_NAMING,
    PLUGIN_NAMING
)

class Guideline(object):
    def __getattr__(self, name:str):
        if name.startswith('__'):
            return getattr(object, name)

        tp = name.split('check_')[-1].upper()
        def func(*args, **kwargs):
            for t in globals()[tp]:
                self.check(*t, *args, **kwargs)

        return func

    @staticmethod
    def check(test, message, subject) -> None:
        if test(subject):
            raise TypeError('{}: {}'.format(message, subject))



import os
import subprocess
import yaml
from pathlib import Path

from savitrigen.config import CodegenConfig

class Bootstrap():
    @staticmethod
    def clone_repo(config:CodegenConfig):
        cwd = Path('source')
        if not Path(cwd / Path('.gitignore')).exists():
            process = subprocess.Popen([
                'git',
                'clone',
                config.boilerplate_repo_url,
                str(cwd)
            ])

            stdout, stderr = process.communicate()

        os.chdir(str(cwd))
        return cwd

    @staticmethod
    def install():
        if Path(Path.cwd() / Path('node_modules')).exists():
            return

        os.system('sh -c "npm install"')
        os.system('sh -c "cd node_modules/\@savitri/frontend && npm install"')




#!/usr/bin/env python3

import os
import runpy

from setuptools import setup

def get_version():
    filename = os.path.join(os.path.dirname(__file__), 'savitrigen', '__init__.py')
    var = runpy.run_path(filename)
    return var['__version__']

_VERSION = get_version()

with open('README.rst', 'r', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='savitrigen',
    version=_VERSION,
    license='ISC',
    author='João Santos',
    author_email='joaosan177@gmail.com',
    description='Savitri code generator',
    loing_description=long_description,
    long_description_content_type='text/x-rst',
    packages=[
        'savitrigen',
        'savitrigen.config',
        'savitrigen.guideline',
        'savitrigen.template',
        'savitrigen.tree'
    ],
    install_requires=[
        'multipledispatch==0.6.0',
        'pyyaml==6.0'
    ]
)



#!/usr/bin/env python3

_REQUIRED_EXECUTABLES = [
    'npm',
    'nodemon'
]

_MISSING_EXECUTABLE_ERROR = """The following executables are required to be accessible through $$PATH in order to run this program:
{}

The following one is missing: {}
"""

class MissingExecutableException(Exception):
    def __init__(self, missing:str):
        self.missing = missing
    def __str__(self):
        return _MISSING_EXECUTABLE_ERROR.format(
            ', '.join(_REQUIRED_EXECUTABLES),
            self.missing
        )

if __name__ == '__main__':
    import shutil
    import sys

    try:
        for n in _REQUIRED_EXECUTABLES:
            if shutil.which(n) is None:
                raise MissingExecutableException(n)
    except MissingExecutableException as e:
        print(e)
        sys.exit(1)

    from savitrigen.commandline import main
    main()



import yaml
import shutil
import os

import savitrigen.logging
from savitrigen.tree.frontend import FrontendTree
from savitrigen.tree.backend import BackendTree
from savitrigen.tree.project import ProjectTree
from savitrigen.bootstrap import Bootstrap
from savitrigen.config import (
    CodegenConfig,
    ProjectConfig,
    BackendConfig,
    FrontendConfig
)

def main():
    config_path = os.path.join(os.path.expanduser('~'), '.savitrigen')
    script_path = '/'.join(os.path.realpath(__file__).split('/')[:-1])

    if not os.path.isdir(config_path):
        os.mkdir(config_path)
        shutil.copyfile(
            '{}/{}'.format(script_path, 'config.yml'),
            '{}/{}'.format(config_path, 'config.yml')
        )

    with open('{}/{}'.format(config_path, 'config.yml'), 'r') as f:
        codegen_config = CodegenConfig(**yaml.safe_load(f))

    with open('briefing.yml', 'r') as f:
        project_config = ProjectConfig(**yaml.safe_load(f))
        backend_config = BackendConfig(
            **project_config.backend,
            plugins=project_config.plugins
        )

        frontend_config = FrontendConfig(
            **project_config.frontend,
            meta=project_config.meta,
            entities=backend_config.entities,
            plugins=project_config.plugins
        )

    Bootstrap.clone_repo(codegen_config)

    ProjectTree(project_config).create()
    BackendTree(backend_config).create()
    FrontendTree(frontend_config).create()

    Bootstrap.install()
